package bug

import (
	"fmt"
	"bug/indirect"
)

// By reading of the language spec, initializeMe should
// always be called first, regardless of any initializations
// that follow. However, with go1.17 if the breaksIfThisExists
// initialization below is present then it will be initialized
// before initializeMe is called.
var _ = initializeMe()

type dummy string

func (d dummy) Get() int {
	if globalX == nil {
		panic("initializeMe was not called\n")
	}
	return *globalX
}

const d = dummy("A")

// If this initialization is present, then initializeMe
// is not called before initialization of breaksIfThisExists. 
// go 1.16 and all prior versions work as expected.
var breaksIfThisExists = struct {
	vals	[]int
} {
	[]int{indirect.GetVal(d)},
}

// This code works as expected, ie. initializeMe is
// called before
var worksWithThis = d.Get()
var _ = indirect.GetVal

var globalX *int

func initializeMe() struct{} {
	fmt.Printf("initMe....\n")
	globalX = new(int)
	*globalX = 33
	return struct{}{}
}

